#include "alu.h"

/*
 *	Exception
 */

const char* parseException::what() const throw() {
	return text.c_str();
}

parseException::parseException(string message) {
	text = message;
}

parseException::~parseException() throw() { }

/*
 *	ALU
 */

ALU::ALU(char *filename) {
	json_object *scheme = json_object_from_file(filename);
	if(scheme == (struct json_object*)error_ptr(-1)) throw parseException("can't read input file");

	json_object *alu_scheme = json_object_object_get(scheme, "ALU");
	if(alu_scheme == NULL) throw parseException("can't find object ALU");

	int arrlen = json_object_array_length(alu_scheme);
	for(int i=0; i<arrlen; i++) {
		json_object *element = json_object_array_get_idx(alu_scheme, i);
		json_object *element_id = json_object_object_get(element, "id");
		string e_id(json_object_get_string(element_id));
		if(all.find(e_id) != all.end()) throw parseException("redefenition");
		
		json_object *element_type = json_object_object_get(element, "type");
		string e_type(json_object_get_string(element_type));
		if(e_type == "in") {
			InputItem *tmp = new InputItem(e_id);
			all[e_id] = tmp;
			in.push_back(tmp);
		} else if(e_type == "out") {
			OutputItem *tmp = new OutputItem;
			all[e_id] = tmp;
			out.push_back(tmp);
		} else if(e_type == "multiplexer") {
			MultiplexerItem *tmp = new MultiplexerItem(e_id);
			all[e_id] = tmp;
			logic.push_back(tmp);
		} else if(e_type == "xor" || e_type == "or" || e_type == "and") {
			LogicItem *tmp = new LogicItem;
			all[e_id] = tmp;
			logic.push_back(tmp);
		} else {
			throw parseException("unkonwn type");
		}
	}
	for(int i=0; i<arrlen; i++) {
		json_object *element = json_object_array_get_idx(alu_scheme, i);
		json_object *element_id = json_object_object_get(element, "id");
		string e_id(json_object_get_string(element_id));
		
		json_object *element_type = json_object_object_get(element, "type");
		string e_type(json_object_get_string(element_type));
		if(e_type == "out") {
			json_object *inputs = json_object_object_get(element, "inputs");
			string inputid = string(json_object_get_string(json_object_array_get_idx(inputs, 0)));
			if(all.find(inputid) == all.end()) throw parseException("can't find element for output");
			(*static_cast<OutputItem*>(all[e_id])) = OutputItem(e_id, all[inputid]);
		} else if(e_type == "multiplexer") {
			for(int j=0; j<11; j++) {
				json_object *inputs = json_object_object_get(element, "inputs");
				string inputid = string(json_object_get_string(json_object_array_get_idx(inputs, j)));
				if(inputid == "") {
					(*static_cast<MultiplexerItem*>(all[e_id]))[j] = NULL;
				} else {
					if(all.find(inputid) == all.end()) throw parseException("can't find element for output");
					(*static_cast<MultiplexerItem*>(all[e_id]))[j] = all[inputid];
				}
			}
		} else if(e_type == "xor" || e_type == "or" || e_type == "and") {
			json_object *inputs = json_object_object_get(element, "inputs");
			string inputid1 = string(json_object_get_string(json_object_array_get_idx(inputs, 0)));
			if(all.find(inputid1) == all.end()) throw parseException("can't find element for logic unit");
			string inputid2 = string(json_object_get_string(json_object_array_get_idx(inputs, 1)));
			if(all.find(inputid2) == all.end()) throw parseException("can't find element for logic unit");
			if(e_type == "xor")	
				(*static_cast<LogicItem*>(all[e_id])) = LogicItem(e_id, all[inputid1], all[inputid2], op_xor);
			else if(e_type == "and")
				(*static_cast<LogicItem*>(all[e_id])) = LogicItem(e_id, all[inputid1], all[inputid2], &op_and);
			else 
				(*static_cast<LogicItem*>(all[e_id])) = LogicItem(e_id, all[inputid1], all[inputid2], &op_or);
		}
	}
	// i hope, it works....
	json_object_object_del(scheme, "ALU");
	free(scheme);
}

ALU::ALU(const ALU& other) : in(other.in), out(other.out), logic(other.logic) {}

ALU::~ALU() {
	for(auto i = all.begin(); i != all.end(); ++i) {
		if(i->second) {
			delete i->second;
		}
	}
}

void ALU::calc() {
	for(int i=0; i<in.size(); i++) cout << in[i]->getName() << '\t';
	for(int i=0; i<out.size(); i++) cout << out[i]->getName() << '\t';
	cout << endl;
	dfs(0);
}

void ALU::dfs(int point) {
	if(point == in.size()) {
		for(auto i = all.begin(); i != all.end(); ++i) if(i->second) i->second->setNeedUpdate(true);
		for(int i=0; i<in.size(); i++) in[i]->setNeedUpdate(false);
		for(int i=0; i<out.size(); i++) out[i]->execute();
		for(int i=0; i<in.size(); i++) cout << in[i]->out() << '\t';
		for(int i=0; i<out.size(); i++) cout << out[i]->out() << '\t';
		cout << endl;
		return ;
	}
	for(int q=0; q<2; q++) {
		in[point]->in(q);
		dfs(point+1);
	}
}
