#include "items.h"

/*
 *	Item
 */

Item::Item() : output(false) {}

Item::Item(const Item& other) : name(other.name), output(other.output) {}

Item::Item(const string& name_) : name(name_), output(false) {}

Item& Item::operator=(const Item& other) {
	if(this != &other) {
		name = other.name;
		output = other.output;
	}
	return *this;
}

Item::~Item() {}

/*
 *	InputItem
 */

InputItem::InputItem() : Item() {
	setNeedUpdate(false);
}

InputItem::InputItem(const InputItem& other) : Item(other) {
	setNeedUpdate(false);
}

InputItem::InputItem(const string& name_) : Item(name_) {
	setNeedUpdate(false);
}

InputItem& InputItem::operator=(const InputItem& other) {
	if(this != &other) *(static_cast<Item*>(this)) = other;
	return *this;
}

InputItem::~InputItem() {}

/*
 *	ConnectedItem
 */

ConnectedItem::ConnectedItem() : Item() {}

ConnectedItem::ConnectedItem(const ConnectedItem& other) : Item(other) {}

ConnectedItem::ConnectedItem(const string& name_) : Item(name_) {}

ConnectedItem& ConnectedItem::operator=(const ConnectedItem& other) {
	if(this != &other) *(static_cast<Item*>(this)) = other;
	return *this;
}

ConnectedItem::~ConnectedItem() {}

/*
 *	LogicItem
 */

LogicItem::LogicItem() : ConnectedItem() {}

LogicItem::LogicItem(const LogicItem& other) : ConnectedItem(other), input1(other.input1), input2(other.input2), eval(other.eval) {}

LogicItem::LogicItem(const string& name_, Item* in1, Item* in2, execFunc f) : ConnectedItem(name_), input1(in1), input2(in2), eval(f) {}

LogicItem& LogicItem::operator=(const LogicItem& other) {
	if(this != &other) {
		*(static_cast<ConnectedItem*>(this)) = other;
		input1 = other.input1;
		input2 = other.input2;
		eval = other.eval;
	}
	return *this;
}

LogicItem::~LogicItem() {}

bool LogicItem::execute() {
	if(!(input1->isUpdated())) input1->execute();
	if(!(input2->isUpdated())) input2->execute();
	setOut(eval(input1->out(), input2->out()));
	setNeedUpdate(false);
	return out();
}

/*
 *	OutputItem
 */

OutputItem::OutputItem() : ConnectedItem() {}

OutputItem::OutputItem(const OutputItem& other) : ConnectedItem(other), input(other.input) {}

OutputItem::OutputItem(const string& name_, Item* in) : ConnectedItem(name_), input(in) {}

OutputItem& OutputItem::operator=(const OutputItem& other) {
	if(this != &other) {
		*(static_cast<ConnectedItem*>(this)) = other;
		input = other.input;
	}
	return *this;
}

OutputItem::~OutputItem() {}

bool OutputItem::execute() {
	if(!(input->isUpdated())) input->execute();
	setOut(input->out());
	setNeedUpdate(false);
	return out();
}

/*
 *	MultiplexerItem
 */

MultiplexerItem::MultiplexerItem() : ConnectedItem() {}

MultiplexerItem::MultiplexerItem(const MultiplexerItem& other) : ConnectedItem(other) {
	setInput();
	setSelect();
}

MultiplexerItem::MultiplexerItem(const string& name_) : ConnectedItem(name_) {
	setInput();
	setSelect();
}

MultiplexerItem MultiplexerItem::operator=(const MultiplexerItem& other) {
	if(this != &other) {
		*(static_cast<ConnectedItem*>(this)) = other;
		for(int i=0; i<8; i++) inputs[i] = other.inputs[i];
		for(int i=0; i<3; i++) select[i] = other.select[i];
	}
	return *this;
}

MultiplexerItem::~MultiplexerItem() {}

void MultiplexerItem::setInput(Item* in0, Item* in1, Item* in2, Item* in3, Item* in4, Item* in5, Item* in6, Item* in7) {
	inputs[0] = in0;
	inputs[1] = in1;
	inputs[2] = in2;
	inputs[3] = in3;
	inputs[4] = in4;
	inputs[5] = in5;
	inputs[6] = in6;
	inputs[7] = in7;
}

void MultiplexerItem::setSelect(Item* sel0, Item* sel1, Item* sel2) {
	select[0] = sel0;
	select[1] = sel1;
	select[2] = sel2;
}

bool MultiplexerItem::execute() {
	for(int i=0; i<8; i++) if(inputs[i] && !(inputs[i]->isUpdated())) inputs[i]->execute();
	int id = 0;
	for(int i=0; i<3; i++) if(select[i]) id |= ((select[i]->out())<<i);
	setOut(inputs[id] ? inputs[id]->out() : false);
	setNeedUpdate(false);
	return out();
}
