#pragma once

#include <string>
#include <iostream>

using std::string;
using std::cerr;
using std::endl;

#define Eo(x) { cerr << #x << " = " << (x) << endl; }

class Item {
	public:
		Item();
		Item(const Item& );
		Item(const string& );
		Item& operator=(const Item& );
		virtual ~Item();

		bool out() const { return output; }
		void setOut(bool value) { output = value; }
		string getName() const { return name; }

		bool isUpdated() const { return !needUpdate; }
		void setNeedUpdate(bool value) { needUpdate = value; }
		virtual bool execute() = 0;

	private:
		string name;
		bool output;
		bool needUpdate;
};

class InputItem : public Item {
	public:
		InputItem();
		InputItem(const InputItem& );
		InputItem(const string& );
		InputItem& operator=(const InputItem&);
		virtual ~InputItem();
		
		void in(bool value) { setOut(value); }
		
		virtual bool execute() { return out(); }
};

class ConnectedItem : public Item {
	public:
		ConnectedItem();
		ConnectedItem(const ConnectedItem& );
		ConnectedItem(const string& );
		ConnectedItem& operator=(const ConnectedItem&);
		virtual ~ConnectedItem();

		virtual bool execute() = 0;
};

typedef bool (*execFunc)(bool, bool);

class LogicItem : public ConnectedItem {
	public:
		LogicItem();
		LogicItem(const LogicItem& );
		LogicItem(const string&, Item*, Item*, execFunc);
		LogicItem& operator=(const LogicItem&);
		virtual ~LogicItem();
		
		virtual bool execute();

	private:
		Item *input1, *input2;
		execFunc eval;
};

class OutputItem : public ConnectedItem {
	public:
		OutputItem();
		OutputItem(const OutputItem& );
		OutputItem(const string&, Item* );
		OutputItem& operator=(const OutputItem& );
		virtual ~OutputItem();

		virtual bool execute();
	private:
		Item* input;
};

class MultiplexerItem : public ConnectedItem {
	public:
		MultiplexerItem();
		MultiplexerItem(const MultiplexerItem& );
		MultiplexerItem(const string& );
		MultiplexerItem operator=(const MultiplexerItem& );
		virtual ~MultiplexerItem();

		Item*& operator[](int id) { return id<8 ? inputs[id] : select[id-8]; }

		void setInput(Item* = NULL, Item* = NULL, Item* = NULL, Item* = NULL, Item* = NULL, Item* = NULL, Item* = NULL, Item* = NULL);
		void setSelect(Item* = NULL, Item* = NULL, Item* = NULL);
		virtual bool execute();

	private:
		Item *inputs[8], *select[3];
};
